@@ -38,3 +38,6 @@ try:
 
 except AttributeError:
     raise ImproperlyConfigured('correct settings.PYMORPHY_DICTS is required for pymorphy template tags.')
+
+MARKER_OPEN = getattr(settings, 'PYMORPHY_MARKER_OPEN', '\[\[')
+MARKER_CLOSE = getattr(settings, 'PYMORPHY_MARKER_CLOSE', '\]\]')
