@@ -43,13 +43,27 @@ class GramFormTest(unittest.TestCase):
         self.assertTrue(form.get_form_string().count(u'дт') == 1)
         self.assertTrue(len(form.get_form_string()) == (2*3)+2)
 
+    def testMatch(self):
+        form = GramForm(u"мр,ед,имя")
+        self.assertTrue(form.match(GramForm(u"мр")))
+        self.assertTrue(form.match(GramForm(u"ед,мр")))
+
+    def testMatch2(self):
+        form = GramForm(u"мр,ед,имя")
+        self.assertFalse(form.match(GramForm(u"мр,!имя")))
+        self.assertTrue(form.match(GramForm(u"ед,!тв")))
+
 
 class TestMorph(unittest.TestCase):
     morph_ru = get_morph(os.path.join(DICT_PATH, 'ru'))
     morph_en = get_morph(os.path.join(DICT_PATH, 'en'))
 
     def check_norm(self, input, output):
-        self.assertEqual(self.morph_ru.normalize(input), set(output))
+        norm_forms = self.morph_ru.normalize(input)
+        correct_norm_forms = set(output)
+
+        msg = u"[%s] != [%s]" % (u", ".join(norm_forms), u", ".join(correct_norm_forms))
+        self.assertEqual(norm_forms, correct_norm_forms, msg)
 
     def check_norm_en(self, input, output):
         self.assertEqual(self.morph_en.normalize(input), set(output))
@@ -94,7 +108,7 @@ class TestMorph(unittest.TestCase):
         self.check_norm(u'ЕЙ', [u'ОНА'])
         self.check_norm(u'Я', [u'Я'])
         self.check_norm(u'МНЕ', [u'Я'])
-        self.check_norm(u'ЕГО', [u'ОН', u'ОНО', u'ЕГО'])
+        self.check_norm(u'ЕГО', [u'ОН', u'ОНО'])
         self.check_norm(u'ЕМУ', [u'ОН', u'ОНО'])
 
     def testNoBase(self):
@@ -107,7 +121,7 @@ class TestMorph(unittest.TestCase):
     def testPredict(self):
         self.check_norm(u'ТРИЖДЫЧЕРЕЗПИЛЮЛЮОКНАМИ', [u'ТРИЖДЫЧЕРЕЗПИЛЮЛЮОКНА'])
         self.check_norm(u'РАЗКВАКАЛИСЬ',[u'РАЗКВАКАТЬСЯ'])
-        self.check_norm(u'КАШИВАРНЕЕ',[u'КАШИВАРНЫЙ'])
+        self.check_norm(u'КАШИВАРНЕЕ', [u'КАШИВАРНЫЙ', u'КАШИВАРНАЯ', u'КАШИВАРНОЕ'])
         self.check_norm(u'ДЕПЫРТАМЕНТОВ',[u'ДЕПЫРТАМЕНТ'])
         self.check_norm(u'ИЗМОХРАТИЛСЯ',[u'ИЗМОХРАТИТЬСЯ'])
 
@@ -115,6 +129,30 @@ class TestMorph(unittest.TestCase):
         self.check_norm(u'БУТЯВКОЙ',[u'БУТЯВКА']) # и никаких местоимений!
         self.check_norm(u'САПАЮТ',[u'САПАТЬ']) # и никаких местоимений!
 
+    def testFemale(self):
+        self.check_norm(u'КЛЮЕВУ', [u'КЛЮЕВ', u'КЛЮЕВА'])
+        self.check_norm(u'КЛЮЕВА', [u'КЛЮЕВА', u'КЛЮЕВ'])
+
+    def testVerbs(self):
+        self.check_norm(u'ГУЛЯЛ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЛА', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЕТ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЮТ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЛИ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯТЬ', [u'ГУЛЯТЬ'])
+
+    def testVerbProducts(self):
+        self.check_norm(u'ГУЛЯЮЩИЙ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯВШИ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЯ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ГУЛЯЮЩАЯ', [u'ГУЛЯТЬ'])
+        self.check_norm(u'ЗАГУЛЯВШИЙ', [u'ЗАГУЛЯТЬ'])
+
+    def testKeepGender(self):
+        self.check_norm(u'КРАСИВЫЙ', [u'КРАСИВЫЙ'])
+        self.check_norm(u'КРАСИВАЯ', [u'КРАСИВАЯ'])
+        self.check_norm(u'КРАСИВОМУ', [u'КРАСИВЫЙ', u'КРАСИВОЕ'])
+        self.check_norm(u'КРАСИВЫЕ', [u'КРАСИВЫЙ', u'КРАСИВАЯ', u'КРАСИВОЕ'])
 
 
 class TestPluraliseRu(unittest.TestCase):
@@ -148,14 +186,11 @@ class TestPluraliseRu(unittest.TestCase):
     def testInvalidWord(self):
         self.assert_plural(u'123', u'123')
 
-# падающие тесты ============================
-    def testPronouns(self):
-        self.assert_plural(u'Я', u'МЫ')
-
     def testInvalidGraminfo(self):
         self.assert_plural(u'НАЧАЛО', u'НАЧАЛА', gram_class=u'С')
 
 
+
 class TestInflectRu(unittest.TestCase):
     morph = get_morph(os.path.join(DICT_PATH, 'ru'))
 
@@ -164,11 +199,45 @@ class TestInflectRu(unittest.TestCase):
         self.assertEqual(morphed_word, result, u"%s != %s" % (morphed_word, result))
 
     def testInflect(self):
-        self.assert_inflect(u"СУСЛИКОВ", "дт", u"СУСЛИКАМ")
-        self.assert_inflect(u"СУСЛИК", "дт", u"СУСЛИКУ")
-        self.assert_inflect(u"СУСЛИКА", "дт", u"СУСЛИКУ")
-        self.assert_inflect(u"СУСЛИК", "мн,дт", u"СУСЛИКАМ")
+        self.assert_inflect(u"СУСЛИКОВ", u"дт", u"СУСЛИКАМ")
+        self.assert_inflect(u"СУСЛИК", u"дт", u"СУСЛИКУ")
+        self.assert_inflect(u"СУСЛИКА", u"дт", u"СУСЛИКУ")
+        self.assert_inflect(u"СУСЛИК", u"мн,дт", u"СУСЛИКАМ")
 
     def testVerbs(self):
-        self.assert_inflect(u"ГУЛЯЮ", "прш", u"ГУЛЯЛ")
-        self.assert_inflect(u"ГУЛЯЛ", "нст", u"ГУЛЯЮ")
+        self.assert_inflect(u"ГУЛЯЮ", u"прш", u"ГУЛЯЛ")
+        self.assert_inflect(u"ГУЛЯЛ", u"нст", u"ГУЛЯЮ")
+
+
+class TestPluralizeInflected(unittest.TestCase):
+    morph = get_morph(os.path.join(DICT_PATH, 'ru'))
+
+    def assert_morph(self, word, count, result, *args, **kwargs):
+        morphed_word = self.morph.pluralize_inflected_ru(word, count, *args, **kwargs)
+        self.assertEqual(morphed_word, result, u"%s != %s" % (morphed_word, result))
+
+    def testParrots(self):
+        self.assert_morph(u"ПОПУГАЙ", 1, u"ПОПУГАЙ")
+        self.assert_morph(u"ПОПУГАЙ", 2, u"ПОПУГАЯ")
+        self.assert_morph(u"ПОПУГАЙ", 3, u"ПОПУГАЯ")
+        self.assert_morph(u"ПОПУГАЙ", 4, u"ПОПУГАЯ")
+        self.assert_morph(u"ПОПУГАЙ", 5, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 7, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 9, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 0, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 10, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 11, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 12, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 15, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 19, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 21, u"ПОПУГАЙ")
+        self.assert_morph(u"ПОПУГАЙ", 32, u"ПОПУГАЯ")
+        self.assert_morph(u"ПОПУГАЙ", 38, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 232, u"ПОПУГАЯ")
+        self.assert_morph(u"ПОПУГАЙ", 111, u"ПОПУГАЕВ")
+        self.assert_morph(u"ПОПУГАЙ", 101, u"ПОПУГАЙ")
+
+    def testButyavka(self):
+        self.assert_morph(u"БУТЯВКА", 1, u"БУТЯВКА")
+        self.assert_morph(u"БУТЯВКА", 2, u"БУТЯВКИ")
+        self.assert_morph(u"БУТЯВКА", 5, u"БУТЯВОК")
